#---------------------------------------------
#Manipuulation d'une distribution de Merton
#---------------------------------------------

pmerton=function(m,s,su,l,t,x){
#Fonction de rpartition
  res=0

  for (n in 0:10){
    z=log(x)-(m-s^2/2)*t
    z=z/sqrt(n*su^2+t*s^2)
    y=pnorm(z)*exp(-l*t)*((l*t)^n)/factorial(n)
    res=res+y
    }
  
  return(res)
  }
  
probaRuine=function(m,s,su,l,g){
#Proba de ruine avec un chargement g
  pmerton(m,s,su,l,1,1/(1+g))

  }
  
marge=function(m,s,su,l,p){
#Dtermination du chargement de scurit par dichotomie
	epsilon=10^-4

	xR=10
	xL=0

  	while ((xR-xL)>epsilon) {
	  	xM=(xR+xL)/2
  		zL=probaRuine(m,s,su,l,xL)-p
      zM=probaRuine(m,s,su,l,xM)-p
      if (zL*zM>0) {xL=xM} else {xR=xM}
		}

	return(xR)

  }
  
margeNormale=function(m,s,p){
#Calcul exact en l'absence de saut pour contrle
  y=m-s^2/2+s*qnorm(p)
  y=exp(-y)-1
  return(y)

  }

putBS=function(r,sigma,K,T){
#Formule de B&S pour le prix d'une option de vente
  d1=(log(1/K)+(r+sigma^2/2)*T)/(sigma*sqrt(T))
  d2=(log(1/K)+(r-sigma^2/2)*T)/(sigma*sqrt(T))
  res=exp(-r*T)*K*pnorm(-d2)-pnorm(-d1)

  return(res)

  }
  
putMerton=function(r,s,su,l,T,K){
#Calcul du prix de l'option de vente dans le modle de Merton
  res=0

  for (n in 0:10){

    sigma=sqrt(n*su^2+T*s^2)
    y=putBS(r,sigma,K,T)*exp(-l*T)*((l*T)^n)/factorial(n)
    res=res+y
    }
  
  return(res)
  }

#---------------------------------------------
#Application numrique

p=.005
mu=log(1.06)
r=log(1.02)
lambda=1.5
v=.16^2
T=5
K=1


#Impact sur le SCR
getMarge=function(a){
#Marge en fonction de la part de variance explique par la composante  sauts (en % du CPR)
  sigmau=sqrt(v*a/lambda)
  sigma=sqrt(v*(1-a))

  res=marge(mu,sigma,sigmau,lambda,p)
  return(res)
  
  }

xPlage=seq(from=0,to=1,by=.01)
resSCR=apply(as.matrix(xPlage),1,getMarge)
plot(x=xPlage,y=resSCR,type="l",xlab="alpha",ylab="Marge (en % du CPR)",col="red")
title("Marge en fonction de la part de variance explique par les sauts")

SCR_CaC40=1/(1+margeNormale(.02,.228,p))-1

#Impact sur les provisions
getPrixOption=function(a){
#Prix de l'option en fonction de la part de variance explique par la composante  sauts
  sigmau=sqrt(v*a/lambda)
  sigma=sqrt(v*(1-a))

  res=putMerton(r,sigma,sigmau,lambda,T,K)
  return(res)

  }
  
resPM=apply(as.matrix(xPlage),1,getPrixOption)
x11()
plot(x=xPlage,y=resPM,type="l",xlab="alpha",ylab="Provision",col="red")
title("Provision en fonction de la part de variance explique par les sauts")
