#---------------------------------------
#Fonctions pour le modle de Vasicek
#---------------------------------------
#Le prix du ZC est donn par P(t,T)=A(T-t)exp(-B(T-t)*r(t))
#On suppose que sous la proba risque neutre la dynamique du taux court est
#dr(t)=a(b-r(t))dt+s*dB(t)

#Coefficient B
getB=function(a,x){
  y=(1-exp(-a*x))/a
  return(y)
  }
  
#Coefficient A
getA=function(a,s,b,x){
  y=exp((b-s^2/(2*a^2))*(getB(a,x)-x)-s^2/(4*a)*getB(a,x)^2)
  return(y)
  }

#Prix du ZC
P=function(a,s,b,r,x){
  y=getA(a,s,b,x)*exp(-getB(a,x)*r)
  return(y)
  }
  
#Prix de march du risque
getLambda=function(a,s,b,x,r,P){
   y=getB(a,x)*(r+s^2*getB(a,x)/(4*a))+log(P)
   y=-y/(getB(a,x)-x)+b-s^2/(2*a^2)
   y=y*a/s
   return(y)
  }

#Moments du taux court
muTaux=function(r,a,b){
  y=r*exp(-a)+b*(1-exp(-a))
  return(y)
  }
  
sigmaTaux=function(a,s){
  y=s*sqrt(getB(2*a,1))
  return(y)
  }

#Simulation du taux court  un an
simuleTaux=function(r,a,s,b,n){
  y=muTaux(r,a,b)+sigmaTaux(a,s)*rnorm(n,0,1)
  return(y)
  }

#Simulation de trajectoires de taux court
simuleTrajectoireTaux=function(r,a,s,b,T,n){
  epsilon=matrix(data=rnorm(n*T),nrow=n,ncol=T)
  rSimule=matrix(data=r,nrow=n,ncol=T+1)
  for (t in 1:T){
    rSimule[,t+1]=muTaux(rSimule[,t],a,b)+sigmaTaux(a,s)*epsilon[,t]
  }
  return(rSimule)
  }
